/*
 * Copyright (C) 2007 by Quentin Anciaux
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *	@author Quentin Anciaux
 */
package org.allcolor.yahp.cl.converter;
import org.allcolor.xml.parser.CXmlParser;

import org.allcolor.yahp.cl.converter.oo.COOoHtmlRenderer;
import org.allcolor.yahp.converter.IHtmlToPdfTransformer;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringWriter;

import java.util.List;
import java.util.Map;


/**
 * This class transform an html document in a PDF.
 *
 * @author Quentin Anciaux
 * @version 0.94
 */
public final class CHtmlToPdfOOoTransformer
	implements IHtmlToPdfTransformer {
	/**
	 * Creates a new CHtmlToPdfOOoTransformer object.
	 */
	public CHtmlToPdfOOoTransformer() {}

	/**
	 * Transform the html document in the inputstream to a pdf in the
	 * outputstream
	 *
	 * @param in html document stream
	 * @param urlForBase base url of the document
	 * @param size not used ! not supported !
	 * @param hf not used ! not supported !
	 * @param properties transform properties
	 * @param out out stream to the pdf file
	 *
	 * @throws CConvertException if a conversion error occured
	 */
	public final void transform(
		final InputStream  in,
		final String	   urlForBase,
		final PageSize	   size,
		final List		   hf,
		final Map		   properties,
		final OutputStream out)
		throws CConvertException {
		try {
			String file = loadFile(in, urlForBase);
			COOoHtmlRenderer.renderToPDF(size, file, urlForBase,
				properties, out);
		} // end try
		catch (final CConvertException e) {
			throw e;
		} // end catch
		catch (final Exception e) {
			throw new CConvertException(
				"ERROR: An unhandled exception occured: " +
				e.getMessage(),
				e);
		} // end catch
		finally {
			try {
				out.flush();
			} // end try
			catch (final Exception ignore) {}
		} // end finally
	} // end transform()

	/**
	 * Set the Content-Type meta tag
	 *
	 * @param fin content of the file
	 *
	 * @return content of the file + the meta tag set
	 */
	public String setCharset(final String fin) {
		String in = fin;

		if (in.indexOf("<head>") != -1) {
			String before = in.substring(0, in.indexOf("<head>") + 6);
			String after = in.substring(in.indexOf("<head>") + 6);
			in = before +
				"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" +
				after;
		} // end if

		return in;
	} // end setCharset()

	/**
	 * Load the file from an inputstream
	 *
	 * @param in the stream to load
	 * @param urlForBase the base url of the document
	 *
	 * @return the content of the stream in a string
	 *
	 * @throws IOException if an error occurs
	 */
	private String loadFile(
		final InputStream in,
		final String	  urlForBase)
		throws IOException {
		Reader		 r		  = CXmlParser.getReader(in);
		char		 buffer[] = new char[2048];
		int			 inb	  = 0;
		StringWriter writer   = new StringWriter();

		while ((inb = r.read(buffer)) != -1) {
			writer.write(buffer, 0, inb);
		} // end while

		String result = writer.toString();
		result     = removeBaseIfPresent(result);
		result     = setCharset(result);

		if (result.indexOf("<head>") != -1) {
			String before = result.substring(0,
					result.indexOf("<head>") + 6);
			String after = result.substring(result.indexOf("<head>") +
					6);
			result = before + "<base href=\"" + urlForBase + "\" />" +
				after;
		} // end if

		return result;
	} // end loadFile()

	/**
	 * Set the Content-Type meta tag
	 *
	 * @param fin content of the file
	 *
	 * @return content of the file + the meta tag set
	 */
	private String removeBaseIfPresent(final String fin) {
		String in    = fin;
		int    index = in.indexOf("<base");

		if (index == -1) {
			return in;
		} // end if

		StringBuffer buffer = new StringBuffer();
		buffer.append(in.substring(in.indexOf("<base")));
		in		  = in.substring(in.indexOf("<base") + 5);
		index     = in.indexOf("/>");

		if (index != -1) {
			buffer.append(in.substring(index + 2));

			return buffer.toString();
		} // end if

		if (index == -1) {
			index = in.indexOf("</base>");
		} // end if

		if (index != -1) {
			buffer.append(in.substring(index + 7));

			return buffer.toString();
		} // end if

		return fin;
	} // end removeBaseIfPresent()
} // end CHtmlToPdfOOoTransformer
