/*
 * Copyright (C) 2007 by Quentin Anciaux
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *	@author Quentin Anciaux
 */

package org.allcolor.yahp.cl.converter;

import org.allcolor.xml.parser.CShaniDomParser;
import org.allcolor.xml.parser.CXmlParser;
import org.allcolor.xml.parser.dom.ADocument;

import org.allcolor.yahp.cl.converter.swing.CSwingHtmlRenderer;
import org.allcolor.yahp.converter.IHtmlToPdfTransformer;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.tidy.Tidy;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringReader;

import java.util.List;
import java.util.Map;


/**
 * This class transform an html document in a PDF.
 *
 * @author Quentin Anciaux
 * @version 0.94
 */
public final class CHtmlToPdfSwingTransformer
    implements IHtmlToPdfTransformer {

    /**
     * Creates a new CHtmlYTLTransformer object.
     */
    public CHtmlToPdfSwingTransformer() {
    }
    
    /**
     * Transform the html document in the inputstream to a pdf in the
     * outputstream
     *
     * @param in html document stream
     * @param urlForBase base url of the document
     * @param size pdf document page size
     * @param hf header-footer list
     * @param properties transform properties
     * @param out out stream to the pdf file
     */
    public final void transform(
        InputStream in,
        String urlForBase,
        PageSize size,
        List hf,
        Map properties,
        OutputStream out
    )  throws CConvertException {
        try {
        	Reader r = CXmlParser.getReader(in);
        	StringBuffer s = new StringBuffer();
        	char [] buffer = new char[2048];
        	int inb = -1;
        	while ((inb = r.read(buffer)) != -1) {
        		s.append(buffer,0,inb);
        	}
        	r.close();
        	Tidy tidy = new Tidy();
        	tidy.setInputEncoding("utf-8");
        	tidy.setXHTML(true);
        	tidy.setQuiet(true);
        	tidy.setShowWarnings(false);
        	tidy.setXmlOut(true);
        	tidy.setIndentContent(true);
        	ByteArrayOutputStream bout = new ByteArrayOutputStream();
        	tidy.parse(new ByteArrayInputStream(s.toString().getBytes("utf-8")), bout);
            CShaniDomParser parser = new CShaniDomParser(true,true);
            Document mydoc = parser.parse(new InputStreamReader(new ByteArrayInputStream(bout.toByteArray()),"utf-8"));
            if (mydoc.toString().length() == 0) {
            	mydoc = parser.parse(new StringReader(s.toString()));
            }
        	try {
        		String surlForBase = ((Element)mydoc.getElementsByTagName("base").item(0)).getAttribute("href");
        		if (surlForBase == null || "".equals(surlForBase)) {
        			surlForBase = null;
        		}
        		if (surlForBase != null) {
        			urlForBase = surlForBase;
        		}
        	}
        	catch (Exception ignore){}
            if (urlForBase != null) {
            	((ADocument) mydoc).setDocumentURI(urlForBase);
            }
            CSwingHtmlRenderer.renderToPDF(
                size,
                mydoc,
                urlForBase,
                hf,
                properties,
                out
            );
        } catch (CConvertException e) {
        	throw e;
        }
        catch (Exception e) {
        	throw new CConvertException("ERROR: An unhandled exception occured: "+e.getMessage(),e);
        } finally {
            try {
                out.flush();
            } catch (Exception ignore) {
            }
        }
    }
}