/*
 * Copyright (C) 2007 by Quentin Anciaux
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *	@author Quentin Anciaux
 */
package org.allcolor.yahp.cl.converter.oo;
import java.net.ConnectException;


/**
 * Simple factory for out-of-container use.
 * @version 0.94
 */
public class COOoConverterFactory {
	/** default port : 8100 */
	public static final int CONNECTION_PORT = 8100;

	/** default hostname : localhost */
	public static final String CONNECTION_HOST = "localhost";

	/** connection to OOo */
	private static COOoConnection unoConnection;

	/** an OOo doc converter handle */
	private static COOoConverter documentConverter;

	/** OOo hostname */
	private static String hostName = CONNECTION_HOST;

	/** OOo port */
	private static int port = CONNECTION_PORT;

	/**
	 * set the hostname of the host on which OOo is running
	 *
	 * @param hostName The hostName to set.
	 */
	public static final synchronized void setHostName(
		final String hostName) {
		COOoConverterFactory.hostName = hostName;
	} // end setHostName()

	/**
 	 * return the hostname of the host on which OOo is running
	 *
	 * @return Returns the hostName.
	 */
	public static final synchronized String getHostName() {
		return hostName;
	} // end getHostName()

	/**
	 * set the port which OOo is running
	 *
	 * @param port The port to set.
	 */
	public static final synchronized void setPort(final int port) {
		COOoConverterFactory.port = port;
	} // end setPort()

	/**
	 * returns the port which OOo is running
	 *
	 * @return Returns the port.
	 */
	public static final synchronized int getPort() {
		return port;
	} // end getPort()

	/**
	 * return an UnoConnection
	 *
	 * @return an UnoConnection
	 *
	 * @throws ConnectException if an error occurs
	 */
	public static synchronized COOoConnection getConnection()
		throws ConnectException {
		if (unoConnection == null) {
			unoConnection = new COOoConnection(port, hostName);
			unoConnection.connect();
		} // end if

		return unoConnection;
	} // end getConnection()

	/**
	 * return a document converter
	 *
	 * @return a document converter
	 *
	 * @throws ConnectException if an error occurs
	 */
	public static synchronized COOoConverter getConverter()
		throws ConnectException {
		if (documentConverter == null) {
			documentConverter = new COOoConverter(getConnection());
		} // end if

		return documentConverter;
	} // end getConverter()
} // end COOoConverterFactory
